/*Copyright (c) 2010 Maxime Petitjean

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.*/

#include "mandelbrot.h"

namespace Plugin
{

Point MandelbrotSequence::init( const Point& _init_point )
{
	m_constant_point = _init_point;
	return _init_point;
}
Point MandelbrotSequence::next( const Point& _point )
{
	return _point * _point + m_constant_point;
}
bool MandelbrotSequence::end( const Point& _point )
{
	return std::norm( _point ) > 4.0;
}

QString MandelbrotFractal::text() const
{
	return	"<p><b>L'ensemble de Mandelbrot</b> est une fractale qui est dfinie comme l'ensemble des points <i>c</i> du plan complexe pour lesquels la suite rcurrente dfinie par : "
			"<i>z</i><sub><i>n</i>+1</sub> = <i>z</i><sub><i>n</i></sub><sup>2</sup> + <i>c</i>"
			" et la condition <i>z</i><sub>0</sub> = 0 <i>ne tend pas</i> vers l'infini (en module).</p>"
			"<p>L'ensemble de Mandelbrot a t cr par <a href=\"http://fr.wikipedia.org/wiki/Benoit_Mandelbrot\">Benot Mandelbrot</a></p>";
}

}

Q_EXPORT_PLUGIN2(mandelbrot, Plugin::MandelbrotFractal)