/*Copyright (c) 2010 Maxime Petitjean

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.*/

#ifndef _NOVA_H_
#define _NOVA_H_

#include "../sequence.h"

/**
 * \brief Gnre une squence de Nova.
 * \image html images/nova.png
 */
class Nova : public Sequence
{
private:
	Point m_constant_point;
	Point m_old_point;

public:
	Nova( const FractalParameters * _fractal_parameters )
		:	Sequence( _fractal_parameters )
	{
	}

protected:
	virtual Point init( const Point& _init_point )
	{
		m_old_point = Point();
		m_constant_point = _init_point;
		return Point( 1.0 );
	}
	virtual Point next( const Point& _point )
	{
		m_old_point = _point;
		return _point - Point(2.0) * ( _point * _point * _point - Point(1.0) ) / ( Point(3.0) * _point * _point ) + m_constant_point;
	}
	virtual bool end( const Point& _point )
	{
		return std::abs( _point - m_old_point ) < 0.05;
	}
};

#endif //_NOVA_H_
