/*Copyright (c) 2010 Maxime Petitjean

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.*/

#include "pluginsview.h"
#include "../plugin/pluginmanager.h"
#include "pluginwidget.h"
#include "pluginsimplewidget.h"

PluginsView::PluginsView( QWidget *_parent )
	:	QListWidget( _parent )
{
	setWindowTitle( tr( "Plugins" ) );
	resize( 500, 250 );
	connect( selectionModel(), SIGNAL( currentChanged( const QModelIndex&, const QModelIndex& ) ), this, SLOT( currentChanged( const QModelIndex&, const QModelIndex& ) ) );
}

PluginsView::~PluginsView()
{
}

void PluginsView::show()
{
	QList< QString > list = PluginManager::instance().plugins();

	clear();
	foreach( QString name, list )
	{
		addItem( QString() );
	}
	for( int i = 0 ; i < list.size() ; i++ )
	{
		QListWidgetItem* widget_item = item( i );
		PluginSimpleWidget* widget = new PluginSimpleWidget( PluginManager::instance().plugins()[i] );
		setItemWidget( widget_item, widget );
		widget_item->setSizeHint( QSize( 0, widget->sizeHint().height() ) );
	}
	if( list.size() > 0 )
		setCurrentRow( 0 );


	QListWidget::show();
}

void PluginsView::currentChanged(  const QModelIndex& _current, const QModelIndex& _previous )
{
	if( _previous.isValid() )
	{
		int row = _previous.row();
		QListWidgetItem* previous_item = item( row );
		PluginSimpleWidget* widget = new PluginSimpleWidget( PluginManager::instance().plugins()[row] );
		setItemWidget( previous_item, widget );
		previous_item->setSizeHint( QSize( 0, widget->sizeHint().height() ) );
	}
	
	if( _current.isValid() )
	{
		int row = _current.row();
		QListWidgetItem* current_item = item( row );
		current_item->setText( QString() );
		PluginWidget* widget = new PluginWidget( PluginManager::instance().plugins()[row] );
		setItemWidget( current_item, widget );
		current_item->setSizeHint( QSize( 0, widget->sizeHint().height() ) );
	}
}
