/*Copyright (c) 2010 Maxime Petitjean

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.*/

#ifndef _FRACTALPLUGIN_H_
#define _FRACTALPLUGIN_H_

#include <QtCore/QString>
#include <QtGui/QImage>
#include <QtPlugin>
#include <complex>

namespace Plugin
{

typedef std::complex< qreal > Point;

class Sequence
{
public:
	virtual Point init( const Point& _init_point ) = 0;
	virtual Point next( const Point& _point ) = 0;
	virtual bool end( const Point& _point ) = 0;
};

class Fractal
{
public:
	enum PointsGeneratorType
	{
		ImageGenerator,
		RandomGenerator
	};
	enum AccumulatorType
	{
		CenterAccumulator,
		SequenceAccumulator
	};

public:
	virtual ~Fractal() {}

	virtual Sequence* sequence() const = 0;
	virtual PointsGeneratorType pointsGeneratorType() const { return ImageGenerator; }
	virtual AccumulatorType accumulatorType() const { return CenterAccumulator; }

	virtual QString name() const = 0;
	virtual QString author() const { return QString(); }
	virtual int version() const { return 0; }
	virtual QImage image() const { return QImage(); }
	virtual QString text() const { return QString(); }
	
};

}

Q_DECLARE_INTERFACE(Plugin::Fractal,
                     "Fractal/1.1")

#endif //_FRACTALPLUGIN_H_
